@@ -23,6 +23,7 @@ from cbw_api_toolbox.__routes__ import ROUTE_PING
 from cbw_api_toolbox.__routes__ import ROUTE_REMOTE_ACCESSES
 from cbw_api_toolbox.__routes__ import ROUTE_SECURITY_ISSUES
 from cbw_api_toolbox.__routes__ import ROUTE_SERVERS
+from cbw_api_toolbox.__routes__ import ROUTE_STORED_CREDENTIALS
 from cbw_api_toolbox.__routes__ import ROUTE_USERS
 from cbw_api_toolbox.cbw_auth import CBWAuth
 
@@ -157,7 +158,7 @@ class CBWApi: # pylint: disable=R0904
 
             return self.verif_response(response)
 
-        logging.error("No server id for update")
+        logging.error("Error: no server id specified")
         return False
 
     def server_schedule_updates(self, server_id, params=None):
@@ -176,7 +177,7 @@ class CBWApi: # pylint: disable=R0904
             response = self._request("DELETE", [ROUTE_SERVERS, server_id])
             return self.verif_response(response)
 
-        logging.error("No server id specific for delete")
+        logging.error("Error: no server id specified for deletion")
         return False
 
     def update_server_cve(self, server_id, cve_code, params=None):
@@ -209,7 +210,7 @@ class CBWApi: # pylint: disable=R0904
             response = self._request("DELETE", [ROUTE_AGENTS, agent_id])
             return self.verif_response(response)
 
-        logging.error("No agent id specific for delete")
+        logging.error("Error: no agent id specified for deletion")
         return False
 
     def remote_accesses(self, params=None):
@@ -241,7 +242,7 @@ class CBWApi: # pylint: disable=R0904
             logging.info('remote access successfully created {}'.format(info["address"]))
             return self._cbw_parser(response)
 
-        logging.error("Error create connection remote access")
+        logging.error("Error: remote access connection could not be created")
         return False
 
     def remote_access(self, remote_access_id):
@@ -262,7 +263,7 @@ class CBWApi: # pylint: disable=R0904
             response = self._request("DELETE", [ROUTE_REMOTE_ACCESSES, remote_access_id])
             return self.verif_response(response)
 
-        logging.error("No remote_access_id for delete")
+        logging.error("Eror: no remote_access id specified for deletion")
         return False
 
     def update_remote_access(self, remote_access_id, info):
@@ -272,7 +273,7 @@ class CBWApi: # pylint: disable=R0904
             logging.debug("Update remote access::{}".format(response.text))
             return self._cbw_parser(response)
 
-        logging.error("Error update remote access")
+        logging.error("Error: the remote access could not be updated")
         return False
 
     def cve_announcement(self, cve_code):
@@ -526,6 +527,14 @@ class CBWApi: # pylint: disable=R0904
             return None
         return self._cbw_parser(response)
 
+    def update_compliance_server(self, server_id, params):
+        """PUT request to /api/v3/compliance/servers/{server_id} to update a compliance asset"""
+        response = self._request("PUT", [ROUTE_COMPLIANCE_ASSETS, server_id], params)
+        if response.status_code != 200:
+            logging.error("Error server id::{}".format(response.text))
+            return None
+        return self._cbw_parser(response)
+
     def recheck_rules(self, server_id):
         """PUT request to /api/v3/compliance/servers/{server_id}/recheck_rules to recheck
         the rules of a specific compliance asset"""
@@ -585,3 +594,45 @@ class CBWApi: # pylint: disable=R0904
             logging.error("Error rule id::{}".format(response.text))
             return None
         return self._cbw_parser(response)
+
+    def stored_credentials(self, params=None):
+        """GET request to /api/v3/assets/credentials to get all stored credentials"""
+        response = self._get_pages("GET", [ROUTE_STORED_CREDENTIALS], params)
+        return response
+
+    def stored_credential(self, credential_id):
+        """GET request to /api/v3/assets/credentials/{credential_id} to retrieve a set of stored credentials"""
+        response = self._request("GET", [ROUTE_STORED_CREDENTIALS, credential_id])
+        if response.status_code != 200:
+            logging.error("Error server id::{}".format(response.text))
+            return None
+        return self._cbw_parser(response)
+
+    def create_stored_credential(self, params):
+        """POST request to /api/v3/credentials to create a stored credentials"""
+        response = self._request("POST", [ROUTE_STORED_CREDENTIALS], params)
+        if response.status_code != 201:
+            logging.error("Error::{}".format(response.text))
+            return None
+
+        return self._cbw_parser(response)
+
+    def update_stored_credential(self, credential_id, params):
+        """PATCH request to /api/v3/credentials/<id> to update a stored credentials"""
+        if  credential_id:
+            response = self._request("PATCH", [ROUTE_STORED_CREDENTIALS, credential_id], params)
+            logging.info("Update stored credential with: {}".format(params))
+            return self.verif_response(response)
+
+        logging.error("Error: No stored crecential id was specified for update")
+        return False
+
+    def delete_stored_credential(self, stored_credentials_id):
+        """DELETE request to /api/v3/assets/credentials/<id> to delete a stored credentials"""
+        if stored_credentials_id:
+            logging.info("Deleting {}".format(stored_credentials_id))
+            response = self._request("DELETE", [ROUTE_STORED_CREDENTIALS, stored_credentials_id])
+            return self.verif_response(response)
+
+        logging.error("Error: no stored_credentials_id was specified for deletion")
+        return False
